﻿

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;
using System.Timers;


namespace LibAppCommon
{
    public class AppLog : ILog
    {
        static AppLog _libLog = null;
        public static AppLog Instance
        {
            get
            {
                if (_libLog != null)
                    return _libLog;
                _libLog = new AppLog();
                return _libLog;
            }
        }

        public static Action<LogMessage> EventLogMessage { get; internal set; }

        ILog _appLog;
        public void SetAppLog(ILog appLog)
        {
            _appLog = appLog;
        }


        Timer _timer = null;
        private AppLog()
        {
            _timer = new Timer();
            _timer.Enabled = true;
            _timer.Interval = 1000; //执行间隔时间,单位为毫秒; 这里实际间隔为10分钟  
            _timer.Start();
            _timer.Elapsed += Timer_Elapsed; ;
        }

        bool _isSavingLog = false;
        private void Timer_Elapsed(object sender, ElapsedEventArgs e)
        {
            if (!_isSavingLog)
            {
                try
                {
                    _isSavingLog = true;
                    SaveLog();
                }
                catch (Exception)
                {
                    System.Diagnostics.Debug.Assert(false);
                }
                _isSavingLog = false;
            }
        }

        Queue<LogMessage> _listLogMessage = new Queue<LogMessage>();
        public void Log(LogMessage message)
        {
            EventLogMessage?.Invoke(message);

            if (_appLog != null)
            {
                _appLog.Log(message);
                return;
            }

            lock (_listLogMessage)
            {
                _listLogMessage.Enqueue(message);
            }
        }


        LogMessage GetNextMessage()
        {
            lock (_listLogMessage)
            {
                if (_listLogMessage.Count == 0)
                    return null;
                return _listLogMessage.Dequeue();
            }
        }

        private void SaveLog()
        {
            while (true)
            {
                LogMessage logMessage = GetNextMessage();
                if (logMessage == null)
                    return;

                try
                {
                    DealLogMessage(logMessage);
                }
                catch (Exception)
                {
                    // System.Diagnostics.Debug.Assert(false);
                }
            }

        }

        public static string GetFileName()
        {
            DateTime now = DateTime.Now;
            string result = string.Format("log_{0}{1}{2}.txt",
                now.Year, now.Month.ToString("00"), now.Day.ToString("00"));
            return result;
        }

        string lastLogFileName = string.Empty;
        FileStream logStream;
        private FileStream GetFileStream()
        {
            string nowFileName = GetFileName();
            if (string.IsNullOrEmpty(lastLogFileName) || nowFileName != lastLogFileName)
            {
                if (logStream != null)
                {
                    logStream.Close();
                }
                logStream = null;
            }

            if (logStream == null)
            {
                lastLogFileName = nowFileName;

                Directory.CreateDirectory("./Log/");
                logStream = new FileStream("./Log/" + lastLogFileName, FileMode.OpenOrCreate);
                logStream.Position = logStream.Length;
            }
            return logStream;
        }

        string GetSaveString(LogMessage logMessage)
        {
            string fileName = Path.GetFileName(logMessage.File);

            if (logMessage.LogType == EN_LogType.信息)
            {
                string result = string.Format("{0}:({1}-{2}-{3})\t*  {4}\r\n",
                        DateTimeToStr(DateTime.Now),
                        fileName, logMessage.Line, logMessage.Member, logMessage.Message);
                return result;
            }
            else
            {
                string result = string.Format("!{0}:({1}-{2}-{3})\t*  {4}\r\n",
                         DateTimeToStr(DateTime.Now),
                         fileName, logMessage.Line, logMessage.Member, logMessage.Message);
                return result;
            }
        }

        public static string DateTimeToStr(DateTime time)
        {
            return time.ToString("yyyy-MM-dd HH:mm:ss");
        }

        private void DealLogMessage(LogMessage logMessage)
        {
            FileStream logStream = GetFileStream();

            string message = GetSaveString(logMessage);
            byte[] buffer = Encoding.UTF8.GetBytes(message);
            logStream.Write(buffer, 0, buffer.Length);
            logStream.Flush();
        }

        //辅助函数
        public static void Info(string txt, bool saveToFile = true, [CallerFilePath] string file = null,
            [CallerLineNumber] int line = 0, [CallerMemberName] string member = null)
        {
            Log(txt, saveToFile, file, line, member, EN_LogType.信息);
        }

        public static void Debug(string txt, bool saveToFile = true, [CallerFilePath] string file = null,
     [CallerLineNumber] int line = 0, [CallerMemberName] string member = null)
        {
            Log(txt, saveToFile, file, line, member, EN_LogType.调试);
        }

        public static void Warning(string txt, bool saveToFile = true, [CallerFilePath] string file = null,
        [CallerLineNumber] int line = 0, [CallerMemberName] string member = null)
        {
            Log(txt, saveToFile, file, line, member, EN_LogType.告警);
        }

        public static void Error(string txt, bool saveToFile = true, [CallerFilePath] string file = null,
            [CallerLineNumber] int line = 0, [CallerMemberName] string member = null)
        {
            Log(txt, saveToFile, file, line, member, EN_LogType.错误);
        }


        public static void Fatal(string txt, bool saveToFile = true, [CallerFilePath] string file = null,
           [CallerLineNumber] int line = 0, [CallerMemberName] string member = null)
        {
            Log(txt, saveToFile, file, line, member, EN_LogType.严重错误);
        }

        public static void Exception(Exception ex, bool saveToFile = true, [CallerFilePath] string file = null,
        [CallerLineNumber] int line = 0, [CallerMemberName] string member = null)
        {
            Log(GetExceptionInfo(ex), saveToFile, file, line, member, EN_LogType.异常, ex);
        }

        static string GetExceptionInfo(Exception ex)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("*********异常文本******");
            sb.AppendLine("【出现时间】：" + DateTime.Now.ToString());
            if (ex != null)
            {
                sb.AppendLine("【异常类型】：" + ex.GetType().Name);
                sb.AppendLine("【异常信息】：" + ex.Message);
                sb.AppendLine("【堆栈调用】：" + ex.StackTrace);
            }
            else
            {
                // sb.AppendLine("【附加信息】：" + backStr);
            }
            sb.AppendLine("*************************");
            return sb.ToString();
        }


        static void Log(string txt, bool saveToFile, string file, int line, string member, EN_LogType type, Exception ex = null)
        {
            LogMessageLib message = new LogMessageLib(type, txt, file, line, member);
            message.Exception = ex;
            AppLog.Instance.Log(message);
        }

    }

    public class LogMessageApp : LogMessage
    {
        public LogMessageApp(EN_LogType logType, string message, [CallerFilePath] string file = null,
          [CallerLineNumber] int line = 0, [CallerMemberName] string member = null)
        {
            Message = message;
            File = file;
            Line = line;
            Member = member;
            LogType = logType;
            ModeName = "app";
        }

        public static new LogMessageApp FromException(Exception ex, [CallerFilePath] string file = null,
            [CallerLineNumber] int line = 0, [CallerMemberName] string member = null)
        {
            LogMessageApp logMessage = new LogMessageApp(EN_LogType.异常, GetExceptionInfo(ex), file, line, member);
            logMessage.Exception = ex;
            logMessage.ModeName = "app";
            return logMessage;
        }
    }

    public class LogMessageLib : LogMessage
    {
        public LogMessageLib(EN_LogType logType, string message, [CallerFilePath] string file = null,
          [CallerLineNumber] int line = 0, [CallerMemberName] string member = null)
        {
            Message = message;
            File = file;
            Line = line;
            Member = member;
            LogType = logType;
            ModeName = "LibCertificate";
        }

        public static new LogMessageLib FromException(Exception ex, [CallerFilePath] string file = null,
            [CallerLineNumber] int line = 0, [CallerMemberName] string member = null)
        {
            LogMessageLib logMessage = new LogMessageLib(EN_LogType.异常, GetExceptionInfo(ex), file, line, member);
            logMessage.Exception = ex;
            logMessage.ModeName = "LibCertificate";
            return logMessage;
        }
    }

}
